# Copyright 2021 Acryl Data, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import re
from typing import List, Optional

from datahub.ingestion.graph.client import DatahubClientConfig, DataHubGraph
from datahub_actions.action.action import Action
from datahub_actions.action.action_registry import action_registry
from datahub_actions.api.action_graph import AcrylDataHubGraph
from datahub_actions.filter.filter import Filter
from datahub_actions.filter.filter_registry import filter_registry
from datahub_actions.pipeline.pipeline_config import (
    ActionConfig,
    FilterConfig,
    FilterSpec,
    SourceConfig,
    TransformConfig,
)
from datahub_actions.pipeline.pipeline_context import PipelineContext
from datahub_actions.plugin.transform.filter.filter_transformer import (
    FilterTransformer,
    FilterTransformerConfig,
)
from datahub_actions.source.event_source import EventSource
from datahub_actions.source.event_source_registry import event_source_registry
from datahub_actions.transform.transformer import Transformer
from datahub_actions.transform.transformer_registry import transformer_registry

logger = logging.getLogger(__name__)


def create_action_context(
    pipeline_name: str, datahub_config: Optional[DatahubClientConfig]
) -> PipelineContext:
    return PipelineContext(
        pipeline_name,
        (
            AcrylDataHubGraph(DataHubGraph(datahub_config))
            if datahub_config is not None
            else None
        ),
    )


def create_event_source(
    source_config: SourceConfig, ctx: PipelineContext
) -> EventSource:
    event_source_type = source_config.type
    event_source_class = event_source_registry.get(event_source_type)
    event_source_instance = None
    try:
        logger.debug(
            f"Attempting to instantiate new Event Source of type {source_config.type}.."
        )
        event_source_config = (
            source_config.config if source_config.config is not None else {}
        )
        event_source_instance = event_source_class.create(event_source_config, ctx)
    except Exception as e:
        raise Exception(
            f"Caught exception while attempting to instantiate Event Source of type {source_config.type}"
        ) from e

    if event_source_instance is None:
        raise Exception(
            f"Failed to create Event Source with type {event_source_type}. Event Source create method returned 'None'."
        )

    return event_source_instance


def create_filter_transformer(
    filter_config: FilterConfig, ctx: PipelineContext
) -> Transformer:
    try:
        logger.debug("Attempting to instantiate filter transformer..")
        filter_transformer_config = FilterTransformerConfig(
            event_type=filter_config.event_type, event=filter_config.event
        )
        return FilterTransformer(filter_transformer_config)
    except Exception as e:
        raise Exception(
            "Caught exception while attempting to instantiate Filter transformer"
        ) from e


def create_filter(filter_spec: FilterSpec, ctx: PipelineContext) -> Filter:
    filter_type = filter_spec.type
    filter_class = filter_registry.get(filter_type)
    filter_instance = None
    try:
        logger.debug(
            f"Attempting to instantiate new Filter of type {filter_spec.type}.."
        )
        filter_config = filter_spec.config if filter_spec.config is not None else {}
        filter_instance = filter_class.create(filter_config, ctx)
    except Exception as e:
        raise Exception(
            f"Caught exception while attempting to instantiate Filter with type {filter_type}"
        ) from e

    if filter_instance is None:
        raise Exception(
            f"Failed to create Filter with type {filter_type}. Filter create method returned 'None'."
        )

    return filter_instance


def create_filters(
    filter_specs: Optional[List[FilterSpec]], ctx: PipelineContext
) -> List[Filter]:
    if filter_specs is None:
        return []
    return [create_filter(spec, ctx) for spec in filter_specs]


def get_filter_name(f: Filter) -> str:
    return type(f).__name__


def create_transformer(
    transform_config: TransformConfig, ctx: PipelineContext
) -> Transformer:
    transformer_type = transform_config.type
    transformer_class = transformer_registry.get(transformer_type)
    transformer_instance = None
    try:
        logger.debug(
            f"Attempting to instantiate new Transformer of type {transform_config.type}.."
        )
        transformer_config = (
            transform_config.config if transform_config.config is not None else {}
        )
        transformer_instance = transformer_class.create(transformer_config, ctx)
    except Exception as e:
        raise Exception(
            f"Caught exception while attempting to instantiate Transformer with type {transformer_type}"
        ) from e

    if transformer_instance is None:
        raise Exception(
            f"Failed to create transformer with type {transformer_type}. Transformer create method returned 'None'."
        )

    return transformer_instance


def create_action(action_config: ActionConfig, ctx: PipelineContext) -> Action:
    action_type = action_config.type
    action_instance = None
    try:
        logger.debug(
            f"Attempting to instantiate new Action of type {action_config.type}.."
        )
        action_class = action_registry.get(action_type)
        action_config_dict = (
            action_config.config if action_config.config is not None else {}
        )
        action_instance = action_class.create(action_config_dict, ctx)
    except Exception as e:
        raise Exception(
            f"Caught exception while attempting to instantiate Action with type {action_type}. "
        ) from e

    if action_instance is None:
        raise Exception(
            f"Failed to create action with type {action_type}. Action create method returned 'None'."
        )

    return action_instance


def normalize_directory_name(name: str) -> str:
    # Lower case & remove whitespaces + periods.
    return re.sub(r"[^\w\-_]", "_", name.lower())


def get_transformer_name(transformer: Transformer) -> str:
    # TODO: Would be better to compute this using the transformer registry itself.
    return type(transformer).__name__



# Copyright 2021 Acryl Data, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from abc import ABCMeta, abstractmethod

from datahub.ingestion.api.closeable import Closeable
from datahub_actions.event.event_envelope import EventEnvelope
from datahub_actions.pipeline.pipeline_context import PipelineContext


class Action(Closeable, metaclass=ABCMeta):
    """
    The base class for all DataHub Actions.

    A DataHub action is a component capable of performing a specific action (notification, auditing, synchronization, & more)
    when important events occur on DataHub.

    Each Action may provide its own semantics, configurations, compatibility and guarantees.
    """

    @classmethod
    @abstractmethod
    def create(cls, config_dict: dict, ctx: PipelineContext) -> "Action":
        """Factory method to create an instance of an Action"""
        pass

    def act(self, event: EventEnvelope) -> bool | None:
        """Take Action on DataHub events, provided an instance of a DataHub event.

        Returns:
            Whether to immediately commit the event as processed successfully.
            Return False if the event has been recorded but should not be acknowledged
            until some later time, e.g. due to batch or asynchronous processing.

            Returning None is equivalent to returning True, i.e. it assumes the event was processed successfully.

        Raises:
            Exception: To indicate the event was not processed successfully and to retry processing the same event.
        """



# Copyright 2021 Acryl Data, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import logging

import pymsteams
from pydantic import SecretStr
from ratelimit import limits, sleep_and_retry

from datahub.configuration.common import ConfigModel
from datahub.metadata.schema_classes import EntityChangeEventClass as EntityChangeEvent
from datahub_actions.action.action import Action
from datahub_actions.event.event_envelope import EventEnvelope
from datahub_actions.pipeline.pipeline_context import PipelineContext
from datahub_actions.utils.datahub_util import DATAHUB_SYSTEM_ACTOR_URN
from datahub_actions.utils.social_util import (
    get_message_from_entity_change_event,
    get_welcome_message,
    pretty_any_text,
)

logger = logging.getLogger(__name__)


@sleep_and_retry
@limits(calls=1, period=1)  # 1 call per second
def post_message(message_card, message):
    message_card.text(message)
    message_card.send()


class TeamsNotificationConfig(ConfigModel):
    webhook_url: SecretStr
    base_url: str = "http://localhost:9002/"
    suppress_system_activity: bool = True


class TeamsNotificationAction(Action):
    def name(self):
        return "TeamsNotificationAction"

    def close(self) -> None:
        pass

    @classmethod
    def create(cls, config_dict: dict, ctx: PipelineContext) -> "Action":
        action_config = TeamsNotificationConfig.model_validate(config_dict or {})
        logger.info(f"Teams notification action configured with {action_config}")
        return cls(action_config, ctx)

    def _new_card(self):
        return pymsteams.connectorcard(
            self.action_config.webhook_url.get_secret_value()
        )

    def __init__(self, action_config: TeamsNotificationConfig, ctx: PipelineContext):
        self.action_config = action_config
        self.ctx = ctx
        welcome_card = self._new_card()
        structured_message = get_welcome_message(self.action_config.base_url)
        welcome_card.title(structured_message.title)
        message_section = pymsteams.cardsection()
        for k, v in structured_message.properties.items():
            message_section.addFact(k, pretty_any_text(v, channel="teams"))
        welcome_card.addSection(message_section)
        post_message(welcome_card, structured_message.text)

    def act(self, event: EventEnvelope) -> None:
        try:
            message = json.dumps(json.loads(event.as_json()), indent=4)
            logger.debug(f"Received event: {message}")
            if event.event_type == "EntityChangeEvent_v1":
                assert isinstance(event.event, EntityChangeEvent)
                if (
                    event.event.auditStamp.actor == DATAHUB_SYSTEM_ACTOR_URN
                    and self.action_config.suppress_system_activity
                ):
                    return None

                semantic_message = get_message_from_entity_change_event(
                    event.event,
                    self.action_config.base_url,
                    self.ctx.graph.graph if self.ctx.graph else None,
                    channel="teams",
                )
                message_card = self._new_card()
                post_message(message_card, semantic_message)
            else:
                logger.debug("Skipping message because it didn't match our filter")
        except Exception as e:
            logger.debug("Failed to process event", e)



PLATFORM_EVENT_TOPIC_NAME = "PlatformEvent_v1"
METADATA_CHANGE_LOG_VERSIONED_TOPIC_NAME = "MetadataChangeLog_Versioned_v1"
METADATA_CHANGE_LOG_TIMESERIES_TOPIC_NAME = "MetadataChangeLog_Timeseries_v1"
ENTITY_CHANGE_EVENT_NAME = "entityChangeEvent"
RELATIONSHIP_CHANGE_EVENT_NAME = "relationshipChangeEvent"



# Copyright 2021 Acryl Data, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Any, Callable

from datahub.metadata.schema_classes import MetadataChangeLogClass
from datahub_actions.event.event_envelope import EventEnvelope
from datahub_actions.event.event_registry import METADATA_CHANGE_LOG_EVENT_V1_TYPE


class MCLProcessor:
    """
    A utility class to register and process MetadataChangeLog events.
    """

    def __init__(self) -> None:
        self.entity_aspect_processors: dict[str, dict[str, Callable]] = {}
        pass

    def is_mcl(self, event: EventEnvelope) -> bool:
        return event.event_type is METADATA_CHANGE_LOG_EVENT_V1_TYPE

    def register_processor(
        self, entity_type: str, aspect: str, processor: Callable
    ) -> None:
        if entity_type not in self.entity_aspect_processors:
            self.entity_aspect_processors[entity_type] = {}
        self.entity_aspect_processors[entity_type][aspect] = processor

    def process(self, event: EventEnvelope) -> Any:
        if isinstance(event.event, MetadataChangeLogClass):
            entity_type = event.event.entityType
            aspect = event.event.aspectName
            if (
                entity_type in self.entity_aspect_processors
                and aspect in self.entity_aspect_processors[entity_type]
            ):
                return self.entity_aspect_processors[entity_type][aspect](
                    entity_urn=event.event.entityUrn,
                    aspect_name=event.event.aspectName,
                    aspect_value=event.event.aspect,
                    previous_aspect_value=event.event.previousAspectValue,
                )



# Copyright 2021 Acryl Data, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from abc import ABCMeta, abstractmethod

from datahub_actions.event.event_envelope import EventEnvelope
from datahub_actions.pipeline.pipeline_context import PipelineContext


class Filter(metaclass=ABCMeta):
    """
    Base class for all DataHub Pipeline Filters.

    A Filter decides whether an event should continue downstream to transformers
    and the action. Returning False from `matches` drops the event.

    Multiple filters in a pipeline are combined with AND semantics: the event
    must satisfy every filter to proceed.
    """

    @classmethod
    @abstractmethod
    def create(cls, config: dict, ctx: PipelineContext) -> "Filter":
        """Factory method to create an instance of a Filter."""
        pass

    @abstractmethod
    def matches(self, event: EventEnvelope) -> bool:
        """Return True if the event should be forwarded downstream."""
        pass



# Copyright 2021 Acryl Data, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

import logging
import time
from dataclasses import dataclass, field
from typing import Any, Dict, Iterable, List, Optional, Sequence

from pydantic import Field

from datahub.configuration import ConfigModel
from datahub.configuration.kafka import _get_schema_registry_url
from datahub.emitter.serialization_helper import post_json_transform
from datahub.metadata.schema_classes import GenericPayloadClass, MetadataChangeLogClass
from datahub.pgqueue.config import (
    PayloadRouteKind,
    PgQueueConnectionConfig,
    PgQueueConsumerConfig,
)
from datahub.pgqueue.consumer import (
    DatahubPgQueueConsumer,
    PgQueueConsumedRecord,
    build_pg_queue_event_meta,
)
from datahub.pgqueue.repository import PgQueueMessageHandle
from datahub_actions.event.event_envelope import EventEnvelope
from datahub_actions.event.event_registry import (
    ENTITY_CHANGE_EVENT_V1_TYPE,
    METADATA_CHANGE_LOG_EVENT_V1_TYPE,
    RELATIONSHIP_CHANGE_EVENT_V1_TYPE,
    MetadataChangeLogEvent,
    RelationshipChangeEvent,
)
from datahub_actions.pipeline.pipeline_context import PipelineContext
from datahub_actions.plugin.source.kafka.kafka_event_source import (
    DEFAULT_TOPIC_ROUTES,
    ENTITY_CHANGE_EVENT_NAME,
    RELATIONSHIP_CHANGE_EVENT_NAME,
    build_entity_change_event,
)
from datahub_actions.source.event_source import EventSource

logger = logging.getLogger(__name__)

_DEFAULT_PG_QUEUE_PAYLOAD_KINDS: Dict[str, PayloadRouteKind] = {
    "mcl": "mcl",
    "mcl_timeseries": "mcl",
    "pe": "pe",
}


class PgQueueEventSourceConfig(ConfigModel):
    queue: PgQueueConnectionConfig
    schema_registry_url: str = Field(default_factory=_get_schema_registry_url)
    schema_registry_config: Dict[str, object] = Field(default_factory=dict)
    topic_routes: Dict[str, str] = Field(
        default_factory=lambda: dict(DEFAULT_TOPIC_ROUTES),
        description="Logical route keys (mcl, pe, …) → logical topic names in pgQueue.",
    )
    payload_kind_by_route_key: Dict[str, PayloadRouteKind] = Field(
        default_factory=lambda: dict(_DEFAULT_PG_QUEUE_PAYLOAD_KINDS),
    )
    visibility_timeout_seconds: Optional[int] = Field(
        default=None,
        description="Override queue visibility timeout when set.",
    )
    poll_interval_seconds: float = Field(default=2.0, ge=0.1, le=120.0)
    batch_size: int = Field(default=100, ge=1, le=5000)

    def build_consumer_config(self, pipeline_name: str) -> PgQueueConsumerConfig:
        return PgQueueConsumerConfig(
            queue=self.queue,
            schema_registry_url=self.schema_registry_url,
            schema_registry_config=dict(self.schema_registry_config),
            topic_routes=dict(self.topic_routes),
            consumer_group=pipeline_name,
            visibility_timeout_seconds=self.visibility_timeout_seconds,
            payload_kind_by_route_key=dict(self.payload_kind_by_route_key),
        )


@dataclass
class PgQueueEventSource(EventSource):
    """Poll pgQueue using metadata-ingestion client; emits the same envelopes as Kafka source."""

    source_config: PgQueueEventSourceConfig = field(init=False)
    ctx: PipelineContext = field(init=False)
    _consumer: Optional[DatahubPgQueueConsumer] = field(
        default=None, init=False, repr=False
    )
    running: bool = field(default=False, init=False)

    def __init__(self, config: PgQueueEventSourceConfig, ctx: PipelineContext):
        self.source_config = config
        self.ctx = ctx
        consumer_cfg = config.build_consumer_config(ctx.pipeline_name)
        self._consumer = DatahubPgQueueConsumer(consumer_cfg)

    @classmethod
    def create(cls, config_dict: dict, ctx: PipelineContext) -> PgQueueEventSource:
        cfg = PgQueueEventSourceConfig.model_validate(config_dict)
        return cls(cfg, ctx)

    def events(self) -> Iterable[EventEnvelope]:
        if self._consumer is None:
            raise RuntimeError("pgQueue consumer is not initialized")

        routes = self.source_config.topic_routes
        preferred = ("mcl", "mcl_timeseries", "pe")
        route_order = [k for k in preferred if k in routes]
        for k in routes:
            if k not in route_order:
                route_order.append(k)

        self.running = True
        logger.info(
            "pgQueue actions source started for pipeline '%s', routes=%s",
            self.ctx.pipeline_name,
            route_order,
        )
        while self.running:
            batch = self._consumer.poll_route_keys(
                route_order, max_messages=self.source_config.batch_size
            )
            if batch:
                for rec in batch:
                    yield from self._records_to_envelopes(rec)
            else:
                time.sleep(self.source_config.poll_interval_seconds)

        logger.info("pgQueue consumer exiting main loop")

    def _records_to_envelopes(
        self, rec: PgQueueConsumedRecord
    ) -> Iterable[EventEnvelope]:
        meta = build_pg_queue_event_meta(rec)
        if isinstance(rec.record, MetadataChangeLogClass):
            mcl_event = MetadataChangeLogEvent.from_class(rec.record)
            yield EventEnvelope(METADATA_CHANGE_LOG_EVENT_V1_TYPE, mcl_event, meta)
            return
        if isinstance(rec.record, dict):
            yield from self._platform_event_envelopes(rec.record, meta)
            return
        logger.warning(
            "Unexpected pgQueue payload type %s for topic %s",
            type(rec.record),
            rec.topic_name,
        )

    def _platform_event_envelopes(
        self, value: Dict[str, Any], meta: Dict[str, Any]
    ) -> Iterable[EventEnvelope]:
        payload: GenericPayloadClass = GenericPayloadClass.from_obj(
            post_json_transform(value["payload"])
        )
        name = value["name"]
        if ENTITY_CHANGE_EVENT_NAME == name:
            ece = build_entity_change_event(payload)
            yield EventEnvelope(ENTITY_CHANGE_EVENT_V1_TYPE, ece, meta)
        elif RELATIONSHIP_CHANGE_EVENT_NAME == name:
            rce = RelationshipChangeEvent.from_json(payload.get("value"))
            yield EventEnvelope(RELATIONSHIP_CHANGE_EVENT_V1_TYPE, rce, meta)

    def ack(self, event: EventEnvelope, processed: bool = True) -> None:
        self.ack_many((event,), processed=processed)

    def ack_many(self, events: Sequence[EventEnvelope], processed: bool = True) -> None:
        """Acknowledge multiple envelopes in one pgQueue round-trip when possible."""
        if not processed or self._consumer is None or not events:
            return
        handles: List[PgQueueMessageHandle] = []
        for event in events:
            pq = event.meta.get("pg_queue") if event.meta else None
            if not pq or "handle" not in pq:
                logger.warning("Missing pg_queue meta; skip ack for one event")
                continue
            handles.append(PgQueueMessageHandle.from_meta_dict(pq["handle"]))
        if handles:
            self._consumer.ack(handles)

    def close(self) -> None:
        self.running = False
        if self._consumer is not None:
            self._consumer.close()
            self._consumer = None



# Copyright 2021 Acryl Data, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import logging
from typing import Any, Dict, List, Optional

from pydantic import Field

from datahub.configuration import ConfigModel
from datahub_actions.event.event_envelope import EventEnvelope
from datahub_actions.filter import match_util
from datahub_actions.filter.filter import Filter
from datahub_actions.pipeline.pipeline_context import PipelineContext

logger = logging.getLogger(__name__)


class EventTypeFilterSpec(ConfigModel):
    """Body predicates for a single event type.

    `event` is a list of body predicate dicts combined with OR semantics:
    the event passes if it satisfies *any* predicate dict.  Within a single
    predicate dict all key/value pairs must match (AND semantics).

    Omitting `event` means "pass on type match alone".
    """

    event: Optional[List[Dict[str, Any]]] = Field(default=None)


class EventTypeFilterConfig(ConfigModel):
    """
    Maps event_type strings to per-type body predicates.

    An event passes this filter when its event_type appears as a key in
    `filter` and satisfies the associated body predicate (if any).

    Example:
        filter:
          MetadataChangeLogEvent_v1:
            event:
              - entityType: schemaField
                aspectName: documentation
              - entityType: dataset
                aspectName: documentation
          EntityChangeEvent_v1:
            event:
              - category: DOCUMENTATION
                entityType: schemaField
    """

    filter: Dict[str, Optional[EventTypeFilterSpec]]


class EventTypeFilter(Filter):
    """
    Passes events whose type is listed in the filter config and whose body
    satisfies the associated predicate (if any).

    Semantics:
    - Across event_type keys: OR — the event must match *any* listed type.
    - Across body predicate list items: OR — the event body must satisfy
      *any* predicate dict in the list.
    - Across keys within a single predicate dict: AND — every key/value pair
      must match.
    """

    def __init__(self, config: EventTypeFilterConfig) -> None:
        self.config = config

    @classmethod
    def create(cls, config_dict: dict, ctx: PipelineContext) -> "EventTypeFilter":
        config = EventTypeFilterConfig.model_validate(config_dict)
        return cls(config)

    def matches(self, event: EventEnvelope) -> bool:
        if event.event_type not in self.config.filter:
            logger.debug(
                "EventTypeFilter: dropping event (type %s not in filter config)",
                event.event_type,
            )
            return False
        spec = self.config.filter[event.event_type]
        if spec is None or spec.event is None:
            logger.debug(
                "EventTypeFilter: passing event (type %s matches, no body predicate)",
                event.event_type,
            )
            return True
        body: Dict[str, Any] = json.loads(event.event.as_json())
        result = any(
            all(match_util.matches(v, body.get(k)) for k, v in predicate.items())
            for predicate in spec.event
        )
        logger.debug(
            "EventTypeFilter: %s event (type %s, body predicate result: %s)",
            "passing" if result else "dropping",
            event.event_type,
            result,
        )
        return result



# Copyright 2021 Acryl Data, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from typing import Type, TypeVar

from datahub.emitter.serialization_helper import post_json_transform
from datahub.metadata.schema_classes import (
    DictWrapper,
    GenericAspectClass,
    GenericPayloadClass,
)

TC = TypeVar("TC", bound=DictWrapper)


def parse_generic_aspect(cls: Type[TC], aspect: GenericAspectClass) -> TC:
    if aspect.contentType != "application/json":
        raise Exception(
            f"Failed to parse aspect. Unsupported content-type {aspect.contentType} provided!"
        )
    return cls.from_obj(post_json_transform(json.loads(aspect.get("value"))))


def parse_generic_payload(cls: Type[TC], payload: GenericPayloadClass) -> TC:
    if payload.contentType != "application/json":
        raise Exception(
            f"Failed to parse payload. Unsupported content-type {payload.contentType} provided!"
        )
    return cls.from_obj(post_json_transform(json.loads(payload.get("value"))))



# Copyright 2021 Acryl Data, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from typing import Any, Dict, List


def matches(match_val: Any, match_val_to: Any) -> bool:
    if isinstance(match_val, dict):
        return matches_dict(match_val, match_val_to)
    if isinstance(match_val, list):
        return matches_list(match_val, match_val_to)
    return match_val == match_val_to


def matches_list(match_filters: List, match_with: Any) -> bool:
    """Any item in match_filters must equal match_with (OR semantics)."""
    if not isinstance(match_with, str):
        return False
    for f in match_filters:
        if f == match_with:
            return True
    return False


def matches_dict(match_filters: Dict[str, Any], match_with: Any) -> bool:
    """All keys in match_filters must match match_with (AND semantics)."""
    if isinstance(match_with, str):
        try:
            match_with = json.loads(match_with)
        except ValueError:
            pass
    if not isinstance(match_with, dict):
        return False
    for key, val in match_filters.items():
        curr = match_with.get(key)
        if not matches(val, curr):
            return False
    return True
